#!/bin/bash
set -euo pipefail
exec 3>&1

FILTER="${1:-}"
: "${DEBUG:=false}"
if "$DEBUG"
then
	REDIR1=/proc/self/fd/1
	REDIR2=/proc/self/fd/2
else
	# can not redirect stdout
	# as behaviour is altered
	# if stdout is tty or not
	REDIR1=/proc/self/fd/1
	REDIR2=/dev/null
fi
DEFAULT_CLEANUP=false
[ "${LOG_ROOT:-}" ] || {
	LOG_ROOT="$(mktemp -d)"
	DEFAULT_CLEANUP=true
}
[ "${CLEANUP+set}" ] || CLEANUP="$DEFAULT_CLEANUP"
TEST_CASE () { {
	if ! "$DEBUG"; then set +x; fi
	if [ "$FILTER" ] && ! [[ "$*" =~ $FILTER ]]
	then
		printf "\x1b[s\x1b[1F\x1b[2CSKIPPED  \x1b[u" >&3
		exit
	fi
	find "$LOG_ROOT" -mindepth 1 -delete
	: "$@"
} 2>"$REDIR2"; }

cleanup () {
	ret="$?"
	set +x
	"$CLEANUP" && rm -rf "$LOG_ROOT"
	if [ "$ret" = "0" ]
	then
		echo "PASS"
		exit 0
	else
		echo "FAIL" >&2
		exit "$ret"
	fi
}
trap 'cleanup' EXIT

RECALL_PREFIX="$(readlink -e ${BASH_SOURCE%/*}/../)"
. ${RECALL_PREFIX}/rc/.bashrc
export LOG_ROOT RECALL_PREFIX
cd "$RECALL_PREFIX"/tests

set -x
{
	RECALL_FZF_ENABLE=false
	RECALL_VIPE_ENABLE=false
	RECALL_ISELECT_ENABLE=false
}
export RECALL_FZF_ENABLE RECALL_VIPE_ENABLE RECALL_ISELECT_ENABLE

: "CLEANUP=$CLEANUP"
: "LOG_ROOT=$LOG_ROOT"

( TEST_CASE "PIPE : Capture : ls : Recall :"
	: Given
	CMD=(ls -lah capture-ls-data)
	: When
	OUT="$(+ "${CMD[@]}")"
	: Then
	[ "$OUT" = "$("${CMD[@]}")" ]
	@ sudo ls && exit 2
	[ "$(@ ls)" = "$("${CMD[@]}")" ]
) >"$REDIR1"

( TEST_CASE "TTY : Capture : ls : Recall :"
	: Given
	CMD=(ls -lah capture-ls-data)
	: When
	+ "${CMD[@]}"
	: Then
	@ sudo ls && exit 2
	[ "$(@ ls)" = "$("${CMD[@]}")" ]
) >"$REDIR1"

( TEST_CASE "PIPE : Capture : ls : Recall : -l"
	: Given
	CMD=(ls -lah capture-ls-data)
	: When
	OUT="$(+ "${CMD[@]}")"
	: Then
	[ "$OUT" = "$("${CMD[@]}")" ]
	@ -ln1 sudo ls && exit 2
	[ "$(@ -ln1 ls)" = "$(find "$LOG_ROOT"/ls -mindepth 1 -maxdepth 1 -print0 | head -z -n1 | xargs -0 readlink -e)" ]
) >"$REDIR1"

( TEST_CASE "TTY : Capture : ls : Recall : -l"
	: Given
	CMD=(ls -lah capture-ls-data)
	: When
	+ "${CMD[@]}"
	: Then
	@ -ln1 sudo ls && exit 2
	[ "$(@ -ln1 ls)" = "$(find "$LOG_ROOT"/ls -mindepth 1 -maxdepth 1 -print0 | head -z -n1 | xargs -0 readlink -e)" ]
) >"$REDIR1"

( TEST_CASE "TTY : Capture : ls : Recall : -sl"
	: Given
	CMD=(ls -lah capture-ls-data)
	: When
	+ "${CMD[@]}"
	: Then
	@ -sln1 sudo ls && exit 2
	[ "$(@ -sln1 ls)" = "${CMD[*]}" ]
) >"$REDIR1"

( TEST_CASE "PIPE : Capture : ls; sudo ls : Recall :"
	: Given
	CMD=(ls -lah capture-ls-data)
	+ "${CMD[@]}"
	CMD_SUDO=(sudo ls -lah capture-ls-data-sudo)
	: When
	OUT="$(+ "${CMD_SUDO[@]}")"
	: Then
	[ "$OUT" = "$("${CMD_SUDO[@]}")" ]
	[ "$(@ sudo ls)" = "$("${CMD_SUDO[@]}")" ]
	[ "$(@ ls)" = "$("${CMD[@]}")" ]
	! [ "$(@ sudo ls)" = "$(@ ls)" ]
) >"$REDIR1"

( TEST_CASE "TTY : Capture : ls; sudo ls : Recall :"
	: Given
	CMD=(ls -lah capture-ls-data)
	+ "${CMD[@]}"
	CMD_SUDO=(sudo ls -lah capture-ls-data-sudo)
	: When
	+ "${CMD_SUDO[@]}"
	: Then
	[ "$(@ sudo ls)" = "$("${CMD_SUDO[@]}")" ]
	[ "$(@ ls)" = "$("${CMD[@]}")" ]
	! [ "$(@ sudo ls)" = "$(@ ls)" ]
) >"$REDIR1"

( TEST_CASE "TTY : Capture : ls; sudo ls : Recall : -sl"
	: Given
	CMD=(ls -lah capture-ls-data)
	+ "${CMD[@]}"
	CMD_SUDO=(sudo ls -lah capture-ls-data-sudo)
	: When
	+ "${CMD_SUDO[@]}"
	: Then
	! [ "$(@ -ln1 sudo ls)" = "$(@ -ln1 ls)" ]
	! [ "$(@ -sln1 sudo ls)" = "$(@ -sln1 ls)" ]
	[ "$(@ -ln1 sudo ls)" = "$(find "$LOG_ROOT"/sudo/ls -mindepth 1 -maxdepth 1 -print0 | head -z -n1 | xargs -0 readlink -e)" ]
	[ "$(@ -ln1 ls)" = "$(find "$LOG_ROOT"/sudo/ls -mindepth 1 -maxdepth 1 -print0 | head -z -n1 | xargs -0 readlink -e)" ]
	[ "$(@ -sln1 sudo ls)" = "${CMD_SUDO[*]}" ]
	[ "$(@ -sln1 ls)" = "${CMD_SUDO[*]}" ]
) >"$REDIR1"
